bl_info = {
    "name": "GP_FromMesh",
    "category": "GPencil",
    "version": (1, 10, 8),
    "blender": (2, 80, 0),
    "author": "Sahin Ersoz",
    "doc_url": "http://ersoz.com/SketchNCarve/SC_Main.html"
}
import bpy
from math import pi
from mathutils import Color
import bmesh
class GP_FromMesh(bpy.types.Operator):
    bl_idname = "gpencil.gp_from_mesh"
    bl_label = "GP_FromMesh"
    bl_options = {'REGISTER', 'UNDO', 'PRESET'}
    process_lines : bpy.props.BoolProperty(name = "Lines", default = 1, description="Process Lines")
    process_fills : bpy.props.BoolProperty(name = "Fills", default = 1, description="Process Faces as fills")
    fix_subdivide : bpy.props.IntProperty(name = "Edge Subdivide:", default = 0, min = 0, max = 100, soft_max = 10, description = "If there are unwanted edges included, try increasing the value.")
    marked_sharp : bpy.props.BoolProperty(name = "Marked_Sharp", default = 0, description="Process Edges Marked Sharp")
    marked_crease : bpy.props.BoolProperty(name = "Marked_Crease", default = 0, description="Process Edges Marked Crease")
    marked_freestyle_edge : bpy.props.BoolProperty(name = "Marked_Freestyle", default = 0, description="Process Edges Marked Freestyle")
    crease_treshold : bpy.props.FloatProperty(name = "Treshold:", default = 0.5, min = 0, max = 1, description ="Crease treshold to be considered as a 'Sharp Edge'.")
    edge_sharpness : bpy.props.FloatProperty(name = "Edge Detection Sharpness:", default = 30, min = 0, max = 180, step = 100, description ="Edge angle treshold to be considered as a 'Sharp Edge'.")
    edge_wobble : bpy.props.FloatProperty(name = "Edge Noise (Amplitude):", default = 0, min = 0, max = 50, soft_min = 0, soft_max = 1, description = "Randomize Amount")
    edge_thickness : bpy.props.FloatProperty(name = "Thickness Noise:", default = 0, min = 0, max = 1, description = "Thickness Noise")
    noise_scale : bpy.props.FloatProperty(name = "Noise Scale (Frequency):", default = 0.5, min = 0, max = 1, description = "Noise Scale")
    noise_randomize : bpy.props.BoolProperty(name = "Randomize", default = 1, description="Use Random Values Over Time")
    noise_random_step : bpy.props.IntProperty(name = "Step", default = 4, min = 1, max = 100, description = "Number of frames before recalculating random values again")
    edge_offset : bpy.props.FloatProperty(name = "Edge Offset:", default = 0.01, step = 1, soft_min = 0, description ="Offset Lines from the surface")
    line_thickness : bpy.props.FloatProperty(name = "Line Thickness:", default = 2, min = 0, max = 1000, soft_max = 10, precision = 3, description = "Adjust Line Thickness")
    hide_mesh : bpy.props.BoolProperty(name = "Hide Mesh", default = 1, description="Hide Mesh Object")
    def draw(self, context):
        layout = self.layout
        box = layout.box()
        row = box.row()
        row.prop(self,"process_lines")
        row.prop(self,"process_fills")
        if self.process_fills == 1:
            row.prop(self,"hide_mesh")
        else:
            row.label(text="")
        if self.process_lines == 1:
            row = box.row()
            row.prop(self,"marked_sharp")
            row = box.row()
            row.prop(self,"marked_crease")
            if self.marked_crease == 1:
                row.prop(self,"crease_treshold")
            row = box.row()
            row.prop(self,"marked_freestyle_edge")
            row = box.row()
            row.prop(self,"fix_subdivide")
            row = box.row()
            row.prop(self,"edge_sharpness")
            row = box.row()
            row.prop(self,"edge_wobble")
            if bpy.app.version >= (2, 83, 0):
                row = box.row()
                row.prop(self,"edge_thickness")
                row = box.row()
                row.prop(self,"noise_scale")
                row = box.row()
                row.prop(self,"noise_randomize")
                if self.noise_randomize == 1:
                    row.prop(self,"noise_random_step")
            row = box.row()
            row.prop(self,"edge_offset")
            row.prop(self,"edge_offset_even")
            row = box.row()
            row.prop(self,"line_thickness")
    def invoke(self, context, event):
        wm = context.window_manager
        return wm.invoke_props_dialog(self)
    def lines(self):
        ob = bpy.context.active_object
        aob = bpy.context.active_object.name
        sx,sy,sz = ob.scale
        ox, oy, oz = ob.location
        rx, ry, rz = ob.rotation_euler
        marked_sharp = self.marked_sharp
        marked_crease = self.marked_crease
        crease_treshold = self.crease_treshold
        marked_freestyle_edge = self.marked_freestyle_edge
        fix_subdivide = self.fix_subdivide
        edge_sharpness = self.edge_sharpness * pi/180
        edge_wobble = self.edge_wobble
        edge_thickness = self.edge_thickness
        noise_scale = self.noise_scale
        noise_randomize = self.noise_randomize
        noise_random_step = self.noise_random_step
        if bpy.app.version < (2, 90, 0):
            edge_offset = -self.edge_offset
        else:
            edge_offset = self.edge_offset
        line_thickness = self.line_thickness
        ob.select_set(True)
        bpy.ops.object.duplicate_move(OBJECT_OT_duplicate={"linked":False, "mode":'TRANSLATION'}, TRANSFORM_OT_translate={"value":(0, 0, 0), "orient_type":'GLOBAL', "orient_matrix":((0, 0, 0), (0, 0, 0), (0, 0, 0)), "orient_matrix_type":'GLOBAL', "constraint_axis":(False, False, False), "mirror":False, "use_proportional_edit":False, "proportional_edit_falloff":'SMOOTH', "proportional_size":1, "use_proportional_connected":False, "use_proportional_projected":False, "snap":False, "snap_target":'CLOSEST', "snap_point":(0, 0, 0), "snap_align":False, "snap_normal":(0, 0, 0), "gpencil_strokes":False, "cursor_transform":False, "texture_space":False, "remove_on_cancel":False, "release_confirm":False, "use_accurate":False})
        dob = bpy.context.active_object
        ob.select_set(False)
        bpy.ops.object.mode_set(mode='EDIT')
        bpy.ops.mesh.select_mode(use_extend=False, use_expand=False, type='VERT')
        bpy.ops.mesh.select_all(action='SELECT')
        bpy.ops.transform.shrink_fatten(value=edge_offset, use_even_offset=False, mirror=True, use_proportional_edit=False, proportional_edit_falloff='SMOOTH', proportional_size=1, use_proportional_connected=False, use_proportional_projected=False, release_confirm=True)
        bpy.ops.mesh.select_all(action='DESELECT')
        bpy.ops.mesh.select_mode(type="EDGE")
        if edge_sharpness >= 0.01:
            bpy.ops.mesh.edges_select_sharp(sharpness=edge_sharpness)
        else:
            bpy.ops.mesh.select_all(action='SELECT')
        if marked_sharp == 1:
            bpy.ops.object.mode_set(mode='OBJECT')
            me = dob.data
            for e in me.edges:
                if e.use_edge_sharp:
                    e.select = True
            bpy.ops.object.mode_set(mode='EDIT')
        if marked_crease == 1:
            me = dob.data
            bpy.ops.object.mode_set(mode='OBJECT')
            for e in me.edges:
                if e.crease > crease_treshold:
                    e.select = True
                    print (e.crease)
            bpy.ops.object.mode_set(mode='EDIT')
        if marked_freestyle_edge == 1:
            bpy.ops.object.mode_set(mode='OBJECT')
            me = dob.data
            for e in me.edges:
                if e.use_freestyle_mark:
                    e.select = True
            bpy.ops.object.mode_set(mode='EDIT')
        try:
            bpy.ops.mesh.select_all(action='INVERT')
            bpy.ops.mesh.delete(type='EDGE')
            bpy.ops.mesh.select_all(action='SELECT')
            bpy.ops.mesh.delete(type='ONLY_FACE')
            bpy.ops.mesh.select_all(action='SELECT')
            if bpy.app.version < (2, 83, 0):
                bpy.ops.mesh.subdivide(number_cuts=10+fix_subdivide, quadcorner='INNERVERT', fractal=edge_wobble, seed=0)
            else:
                bpy.ops.mesh.subdivide(number_cuts=10+fix_subdivide, quadcorner='INNERVERT', fractal=0, seed=0)
            bpy.ops.object.mode_set(mode='OBJECT')
            bpy.ops.object.convert(target='CURVE')
            bpy.ops.object.convert(target='GPENCIL')
            bpy.context.object.name = aob + "_Lines.GP"
            gp = bpy.context.object
            gp.location = ox, oy, oz
            gp.rotation_euler = rx, ry, rz
            gp.data.layers["GP_Layer"].info = "Lines"
            bpy.ops.gpencil.layer_change(layer='Lines')
            objects = bpy.data.objects
            gp.data.stroke_depth_order = '3D'
            col_name = aob+'.lines'
            if bpy.data.materials and col_name in bpy.data.materials:
                new_index = len(gp.material_slots)
                mat = bpy.data.materials[col_name]
                gp.data.materials.append(mat)
            elif len(gp.material_slots) == 1:
                gp.active_material_index = 0
                gp.active_material.name = col_name
                gp.active_material.grease_pencil.color = (0, 0, 0, 1)
                gp.active_material.grease_pencil.show_stroke = True
                gp.active_material.grease_pencil.show_fill = False
                gp.active_material.grease_pencil.mode = 'LINE'
            else:
                new_index = len(ob.material_slots)
                mat = bpy.data.materials.new(name=col_name)
                bpy.data.materials.create_gpencil_data(mat)
                gp.data.materials.append(mat)
                gp.active_material_index = new_index
                gp.active_material.grease_pencil.color = (0, 0, 0, 1)
                gp.active_material.grease_pencil.show_stroke = True
                gp.active_material.grease_pencil.show_fill = False
                gp.active_material.grease_pencil.mode = 'LINE'
        except:
            tavuk = 5
        bpy.ops.object.mode_set(mode='OBJECT')
        bpy.ops.object.select_all(action='DESELECT')
        try:
            dob.select_set(True)
            bpy.ops.object.delete(use_global=False)
        except:
            pass
        if bpy.app.version >= (2, 83, 0):
            bpy.context.view_layer.objects.active = gp
            bpy.ops.object.gpencil_modifier_add(type='GP_NOISE')
            bpy.context.object.grease_pencil_modifiers["Noise"].factor = edge_wobble
            bpy.context.object.grease_pencil_modifiers["Noise"].factor_strength = 0
            bpy.context.object.grease_pencil_modifiers["Noise"].factor_thickness = edge_thickness
            bpy.context.object.grease_pencil_modifiers["Noise"].noise_scale = noise_scale
            try:
                bpy.context.object.grease_pencil_modifiers["Noise"].use_random = noise_randomize
            except:
                bpy.context.object.grease_pencil_modifiers["Noise"].random = noise_randomize
            bpy.context.object.grease_pencil_modifiers["Noise"].step = noise_random_step
            bpy.context.object.grease_pencil_modifiers["Noise"].layer = "Lines"
        bpy.ops.object.gpencil_modifier_add(type='GP_THICK')
        try:
            bpy.context.object.grease_pencil_modifiers["Thickness"].thickness_factor = line_thickness
        except:
            bpy.context.object.grease_pencil_modifiers["Thickness"].thickness = line_thickness
        bpy.context.object.grease_pencil_modifiers["Thickness"].layer = "Lines"
        bpy.ops.object.mode_set(mode='PAINT_GPENCIL')
    def fills(self):
        bpy.ops.object.mode_set(mode='OBJECT')
        obj = bpy.context.active_object
        aob = bpy.context.active_object.name
        sx,sy,sz = obj.scale
        ox, oy, oz = obj.location
        rx, ry, rz = obj.rotation_euler
        gp_mesh=aob + "_Lines.GP"
        hide_mesh = self.hide_mesh
        mesh = obj.data
        if 1>len(mesh.materials):
            m_mat = bpy.data.materials.new(name=aob + "Mat")
            if obj.data.materials:
                obj.data.materials[0] = m_mat
            else:
                obj.data.materials.append(m_mat)
            m_mat.use_nodes = True
        if gp_mesh not in bpy.context.scene.objects:
            bpy.ops.object.gpencil_add(location=(0, 0, 0), type='EMPTY')
            bpy.context.object.data.stroke_depth_order = '3D'
            bpy.context.object.name = gp_mesh
        gpencil = bpy.context.scene.objects[gp_mesh]
        bpy.context.view_layer.objects.active = gpencil
        ob = bpy.context.active_object
        gp_name = gpencil.name
        gp_layer_name='Fills'
        if gpencil.data.layers and gp_layer_name in gpencil.data.layers:
            gpencil_layer = gpencil.data.layers[gp_layer_name]
        else:
            gpencil_layer = gpencil.data.layers.new(gp_layer_name, set_active=True)
        bpy.context.view_layer.objects.active = gpencil
        bpy.ops.gpencil.layer_change(layer='Fills')
        gp_layer = gpencil_layer
        gpencil_layer.clear()
        scene=bpy.data.scenes['Scene']
        bpy.ops.gpencil.layer_change(layer=gp_layer_name)
        gp_frame = gp_layer.frames.new(scene.frame_current)
        mat_list = []
        mat_name_list = []
        matcol_list = []
        for f in obj.data.polygons:
            v = []
            slot = obj.material_slots[f.material_index]
            mat = slot.material
            mat_name = mat.name
            try:
                if mat.use_nodes == True:
                    nodes = mat.node_tree.nodes
                    principled = next(n for n in nodes if n.type == 'BSDF_PRINCIPLED')
                    base_color = principled.inputs['Base Color']
                    value = base_color.default_value
                    color = Color( (value[0], value[1], value[2]) )
                    print (value)
                else:
                    value = mat.diffuse_color
                    color = Color( (value[0], value[1], value[2]) )
                    print (value)
            except:
                value = (0.8000, 0.0000, 0.0000)
                color = Color( (value[0], value[1], value[2]) )
                print (value)
            if mat is not None:
                mat_list.append(slot.material)
                mat_name_list.append(mat_name)
                matcol_list.append(color)
            else:
                print("No mat in slot", f.material_index)
            col_name = mat_name + "gp"
            if bpy.data.materials and col_name in bpy.data.materials:
                if col_name in ob.data.materials:
                    new_index = ob.data.materials.find(col_name)
                    mat_gp = bpy.data.materials[col_name]
                else:
                    new_index = len(ob.material_slots)
                    mat_gp = bpy.data.materials[col_name]
                    ob.data.materials.append(mat_gp)
            else:
                new_index = len(ob.material_slots)
                mat_gp = bpy.data.materials.new(name=col_name)
                bpy.data.materials.create_gpencil_data(mat_gp)
                ob.data.materials.append(mat_gp)
                ob.active_material_index = new_index
                ob.active_material.grease_pencil.color = (0, 0, 0, 1)
                ob.active_material.grease_pencil.fill_color = (value[0], value[1], value[2], 1)
                ob.active_material.grease_pencil.show_stroke = False
                ob.active_material.grease_pencil.show_fill = True
            for idx in f.vertices:
                v.append(obj.data.vertices[idx].co)
            v.append(v[0])
            vlen = (len(v))
            def draw_line(gp_frame, p0: tuple):
                gp_stroke = gp_frame.strokes.new()
                gp_stroke.display_mode = '3DSPACE'
                gp_stroke.line_width = 1
                gp_stroke.material_index = new_index
                ob = bpy.context.active_object
                gp_stroke.points.add(count=vlen)
                for i in range(vlen):
                    gp_stroke.points[i].co = v[i]
                return gp_stroke
            v1 = v[0]
            draw_line(gp_frame, (v1))
        bpy.ops.object.mode_set(mode='EDIT_GPENCIL')
        bpy.ops.gpencil.select_all(action='SELECT')
        bpy.ops.gpencil.stroke_subdivide(only_selected=False)
        bpy.ops.object.mode_set(mode='OBJECT')
        ob.location = ox, oy, oz
        ob.rotation_euler = rx, ry, rz
        ob.scale = sx,sy,sz
        obj.hide_set(hide_mesh)
        obj.hide_render = hide_mesh
        bpy.ops.object.mode_set(mode='PAINT_GPENCIL')
        bpy.context.scene.tool_settings.gpencil_sculpt.lock_axis = 'AXIS_Y'
        bpy.ops.gpencil.layer_change(layer='Fills')
        bpy.ops.gpencil.layer_move(type='DOWN')
        bpy.context.object.active_material_index = 0
    def execute(self, context):
        ob = bpy.context.active_object
        aob = bpy.context.active_object.name
        sx,sy,sz = ob.scale
        ox, oy, oz = ob.location
        rx, ry, rz = ob.rotation_euler
        process_lines = self.process_lines
        process_fills = self.process_fills
        if ob.type == "MESH":
            if process_lines == 1:
                self.lines()
            if process_fills == 1:
                bpy.ops.object.mode_set(mode='OBJECT')
                bpy.context.view_layer.objects.active = ob
                self.fills()
        return {'FINISHED'}
def menu_func(self, context):
    self.layout.operator(GP_FromMesh.bl_idname)
def register():
    bpy.utils.register_class(GP_FromMesh)
    bpy.types.VIEW3D_MT_object.append(menu_func)
def unregister():
    bpy.utils.unregister_class(GP_FromMesh)
    bpy.types.VIEW3D_MT_object.remove(menu_func)
if __name__ == "__main__":
    register()
